(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     44636,       1159]
NotebookOptionsPosition[     42540,       1091]
NotebookOutlinePosition[     43035,       1111]
CellTagsIndexPosition[     42992,       1108]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
How to analyze the convergence rate of a nonlinear solver\
\>", "Title",
 CellChangeTimes->{{3.524754214420076*^9, 3.524754221455688*^9}, {
  3.524754392759589*^9, 3.5247543953179936`*^9}}],

Cell[TextData[{
 "This ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notebook solves a nonlinear function f[x]=0 using three methods:\n\t(a) \
Built-in solver\n\t(b) Explicitly programmed classical Newton-Raphson solver \
\n\t(c) Explicitly programmed Modified Newton-Raphson solver"
}], "Text",
 CellChangeTimes->{{3.5247542252776947`*^9, 3.5247543838363733`*^9}}],

Cell[BoxData[
 RowBox[{"SetDirectory", "[", 
  RowBox[{"NotebookDirectory", "[", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.524757852118265*^9, 3.5247578610882807`*^9}}],

Cell[CellGroupData[{

Cell["Case study", "Section",
 CellChangeTimes->{{3.5247544589661055`*^9, 3.52475447291253*^9}, {
  3.5247549731306086`*^9, 3.52475497412901*^9}, {3.5247563707402635`*^9, 
  3.524756373953869*^9}}],

Cell["Consider", "Text",
 CellChangeTimes->{{3.524753416993475*^9, 3.5247534263378916`*^9}, {
  3.524754401277204*^9, 3.524754415286029*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"f", "[", "x_", "]"}], ":=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"x", "-", "1.5"}], ")"}], 
   RowBox[{"(", 
    RowBox[{"x", "-", "2.3"}], ")"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "3"}], "}"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "3"}], "}"}], ",", "All"}], "}"}]}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}]}], ",", 
   RowBox[{"PlotLabel", "\[Rule]", 
    RowBox[{"Row", "[", 
     RowBox[{"{", 
      RowBox[{"\"\<y=\>\"", ",", 
       RowBox[{"Expand", "[", 
        RowBox[{"f", "[", "x", "]"}], "]"}]}], "}"}], "]"}]}]}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.5247510610389385`*^9, 3.5247511109590263`*^9}, {
  3.524751244738062*^9, 3.5247512480608673`*^9}, {3.5247534301286983`*^9, 
  3.5247538147161736`*^9}}],

Cell["\<\
By design, the exact solutions are x=1.5 and x=2.3\
\>", "Text",
 CellChangeTimes->{{3.524754429794054*^9, 3.524754440433273*^9}}],

Cell[BoxData[
 RowBox[{"xexact", "=", 
  RowBox[{"x", "/.", 
   RowBox[{"Solve", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "x", "]"}], "\[Equal]", "0"}], ",", "x"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5247511991703815`*^9, 3.52475122673563*^9}}],

Cell[CellGroupData[{

Cell["\<\
Convergence analysis for the built-in nonlinear solver\
\>", "Subsection",
 CellChangeTimes->{{3.524756389413496*^9, 3.5247564004115157`*^9}}],

Cell[TextData[{
 "The following applys the built-in solver using a starting guess of x=0.\n\
The Reap and Sow stuff is how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " lets us see the intermediate results, rather than just the final result."
}], "Text",
 CellChangeTimes->{{3.5247565537597847`*^9, 3.5247566126342883`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"res", ",", 
     RowBox[{"{", "xlist", "}"}]}], "}"}], "=", 
   RowBox[{"Reap", "[", 
    RowBox[{"FindRoot", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "x", "]"}], "\[Equal]", "0"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0"}], "}"}], ",", 
      RowBox[{"AccuracyGoal", "\[Rule]", "Infinity"}], ",", 
      RowBox[{"PrecisionGoal", "\[Rule]", "2000"}], ",", 
      RowBox[{"EvaluationMonitor", "\[RuleDelayed]", 
       RowBox[{"Sow", "[", "x", "]"}]}]}], "]"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5247511201786427`*^9, 3.5247511333606663`*^9}, 
   3.5247511642731204`*^9, {3.524751232866441*^9, 3.524751233771242*^9}, {
   3.5247512640508957`*^9, 3.524751290149741*^9}, {3.524751330772213*^9, 
   3.524751332441416*^9}, {3.524751767214179*^9, 3.5247517935470257`*^9}, {
   3.524751826370683*^9, 3.524751856320335*^9}, 3.5247564251219587`*^9, 
   3.5247575019460506`*^9}],

Cell["\<\
The res output is the solution and the xlist array contains the successively \
improved guesses\
\>", "Text",
 CellChangeTimes->{{3.5247564140927396`*^9, 3.5247564483659997`*^9}, 
   3.524757506906859*^9}],

Cell[BoxData["res"], "Input",
 CellChangeTimes->{{3.5247564524220066`*^9, 3.5247564527652073`*^9}}],

Cell[BoxData["xlist"], "Input",
 CellChangeTimes->{{3.524756454855611*^9, 3.524756455370412*^9}, 
   3.524757509746064*^9}],

Cell["\<\
The following table goes from i=1 to Length[xlist]-1 to drop the last error \
(since it typically bucks the convergence trend by essentially repeating the \
result right before it)\
\>", "Text",
 CellChangeTimes->{{3.5247519082684264`*^9, 3.5247520026649914`*^9}, {
   3.5247545764499116`*^9, 3.524754577760314*^9}, 3.5247575126164694`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Select", " ", "which", " ", "of", " ", "the", " ", "two", " ", "exact", 
    " ", "roots", " ", "has", " ", "been", " ", "found"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"iexact", "=", "1"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{
     "Create", " ", "a", " ", "table", " ", "of", " ", "errors", " ", "in", 
      " ", "the", " ", "x", " ", "values", " ", "at", " ", "each", " ", 
      "iteration"}], ",", " ", 
     RowBox[{"but", " ", 
      RowBox[{"don", "'"}], "t", " ", "include", " ", "the", " ", "last", " ",
       "one", " ", "since", " ", 
      RowBox[{"Mathematica", "'"}], "s", " ", "final", " ", "value", " ", 
      "typically", " ", "bucks", " ", "the", " ", "convergence", " ", 
      "trends"}], ",", " ", 
     RowBox[{
     "probably", " ", "from", " ", "some", " ", "unknown", " ", 
      "\"\<clean up\>\"", " ", "step"}]}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ex", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Abs", "[", 
        RowBox[{
         RowBox[{"xlist", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "-", 
         RowBox[{"xexact", "[", 
          RowBox[{"[", "iexact", "]"}], "]"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "xlist", "]"}], "-", "1"}]}], "}"}]}], 
      "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"Create", " ", "a", " ", "list", " ", "of", " ", 
      SubscriptBox["u", "i"]}], "=", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"Log", "[", 
        SubscriptBox["e", "i"], "]"}]}], " ", "values"}]}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"u", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"Log", "[", 
         RowBox[{"ex", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "ex", "]"}], "-", "1"}]}], "}"}]}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"Create", " ", "a", " ", "list", " ", "of", " ", 
      SubscriptBox["v", "i"]}], "=", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"Log", "[", 
        SubscriptBox["e", 
         RowBox[{"i", "+", "1"}]], "]"}]}], " ", "values"}]}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"v", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"Log", "[", 
         RowBox[{"ex", "[", 
          RowBox[{"[", 
           RowBox[{"i", "+", "1"}], "]"}], "]"}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "ex", "]"}], "-", "1"}]}], "}"}]}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"Combine", " ", "the", " ", "lists", " ", "into", " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["u", "i"], ",", 
       SubscriptBox["v", "i"]}], ")"}], " ", "data", " ", "pairs"}], " ", 
    "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"uvdat", "=", 
     RowBox[{"Transpose", "[", 
      RowBox[{"{", 
       RowBox[{"u", ",", "v"}], "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Perform", " ", "a", " ", "linear", " ", "fit", " ", "to", " ", "the", 
     " ", "data"}], "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"model", "=", 
     RowBox[{"LinearModelFit", "[", 
      RowBox[{"uvdat", ",", "ufit", ",", "ufit"}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Create", " ", "a", " ", "plot", " ", "showing", " ", "data", " ", "and", 
     " ", "the", " ", "fit"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Show", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Plot", "[", 
       RowBox[{
        RowBox[{"Normal", "[", "model", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"ufit", ",", 
          RowBox[{"Min", "[", "u", "]"}], ",", 
          RowBox[{"Max", "[", "u", "]"}]}], "}"}], ",", 
        RowBox[{"PlotStyle", "\[Rule]", "Thick"}]}], "]"}], ",", 
      RowBox[{"ListPlot", "[", 
       RowBox[{"uvdat", ",", 
        RowBox[{"PlotStyle", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"PointSize", "[", "0.02", "]"}], ",", "Red"}], "}"}]}]}], 
       "]"}], ",", 
      RowBox[{"AxesLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(u\), \
\(i\)]\)=-Log[\!\(\*SubscriptBox[\(e\), \(i\)]\)]\>\"", ",", 
         "\"\<\!\(\*SubscriptBox[\(v\), \
\(i\)]\)=-Log[\!\(\*SubscriptBox[\(e\), \(i + 1\)]\)]\>\""}], "}"}]}], ",", 
      RowBox[{"PlotLabel", "\[Rule]", 
       RowBox[{"Row", "[", 
        RowBox[{"{", 
         RowBox[{"\"\<\!\(\*SubscriptBox[\(v\), \(i\)]\)=\>\"", ",", 
          RowBox[{
           RowBox[{"Normal", "[", "model", "]"}], "/.", 
           RowBox[{
           "ufit", "\[Rule]", 
            "\"\<\!\(\*SubscriptBox[\(u\), \(i\)]\)\>\""}]}]}], "}"}], 
        "]"}]}]}], "]"}], "//", "Framed"}]}]}]], "Input",
 CellChangeTimes->{{3.524751421845173*^9, 3.5247515344305706`*^9}, {
   3.52475157977985*^9, 3.524751599045884*^9}, {3.524751629341137*^9, 
   3.524751650135974*^9}, {3.524752038279854*^9, 3.5247520562510853`*^9}, {
   3.524752111740383*^9, 3.5247521447968407`*^9}, {3.5247522010973396`*^9, 
   3.524752361262821*^9}, {3.524752395910482*^9, 3.524752447437372*^9}, {
   3.5247524824438343`*^9, 3.5247527195330505`*^9}, {3.524752820760428*^9, 
   3.5247528252376356`*^9}, {3.524752945342247*^9, 3.524752946496649*^9}, {
   3.524752976698302*^9, 3.5247530522180347`*^9}, {3.5247531034017243`*^9, 
   3.5247533886482253`*^9}, {3.524753839504617*^9, 3.524753950826413*^9}, 
   3.5247541166235037`*^9, {3.524757513568071*^9, 3.5247577489708843`*^9}}],

Cell["\<\
Based on the plot label, the slope of the fitted line is 1.9, indicating that \
the built-in solver is approximately second-order accurate.\
\>", "Text",
 CellChangeTimes->{{3.5247541242207174`*^9, 3.524754193516039*^9}, {
  3.5247573487693815`*^9, 3.5247573825746403`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Convergence analysis for the hand-coded \[OpenCurlyDoubleQuote]dumb\
\[CloseCurlyDoubleQuote] CNR solver\
\>", "Subsection",
 CellChangeTimes->{{3.524756389413496*^9, 3.5247564004115157`*^9}, {
  3.5247564948384814`*^9, 3.5247565109845095`*^9}}],

Cell[TextData[{
 "The following is a brute force implmentation that has no protections \
against division by zero from a bad first guess, nor any scaling to ensure \
that built-in tolerance is reasonable.  Constitutive models are a good \
example of the danger of forgetting to scale. For a constitutive model, the \
stress is a function of strain. Stress typically has values on the order of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["10", "9"], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ", which would make the tolerance of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["10", 
    RowBox[{"-", "8"}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " in the following code overly agressive, possibly leading to an infinite \
loop of trying to find a solution that can\[CloseCurlyQuote]t be found with \
such tight tolerances.  On the other hand, if this solver were to be applied \
to a problem where f[x] gives a strain value, then it might converge \
prematurely since strain values are often very tiny numbers.   Lesson: know \
the limitations of the numerical solvers, and perform scaling or other \
protective actions accordingly.  "
}], "Text",
 CellChangeTimes->{{3.524755670642234*^9, 3.52475602722766*^9}, {
  3.524756218437196*^9, 3.5247562637552757`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dumbCNRsolver", "[", 
   RowBox[{"f_", ",", "xguess_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"xlist", ",", 
      RowBox[{"tolerance", "=", 
       RowBox[{"1.0", " ", 
        SuperscriptBox["10", 
         RowBox[{"-", "8"}]]}]}], ",", "x"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"xlist", "=", 
      RowBox[{"{", "xguess", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"x", "=", "xguess"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"f", "[", "x", "]"}], "]"}], ">", "tolerance"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "=", 
         RowBox[{"x", "-", 
          FractionBox[
           RowBox[{"f", "[", "x", "]"}], 
           RowBox[{
            RowBox[{"f", "'"}], "[", "x", "]"}]]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"xlist", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"xlist", ",", "x"}], "]"}]}], ";"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Return", "[", "xlist", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.524754978809018*^9, 3.524755000617857*^9}, {
  3.524755039571125*^9, 3.5247552465834885`*^9}, {3.5247552929623704`*^9, 
  3.524755426467405*^9}, {3.5247554761066923`*^9, 3.524755490131117*^9}, {
  3.524755551161624*^9, 3.5247555554516315`*^9}, {3.524755598492107*^9, 
  3.5247556423437843`*^9}, {3.5247556897210674`*^9, 3.5247556918582706`*^9}, {
  3.524756036587676*^9, 3.524756037476878*^9}, {3.52475620913958*^9, 
  3.5247562103095818`*^9}, {3.5247563185425715`*^9, 3.5247563191665726`*^9}}],

Cell["\<\
Apply the solver to get a list of the x values for each iteration:\
\>", "Text",
 CellChangeTimes->{{3.5247574349283323`*^9, 3.524757453866766*^9}}],

Cell[BoxData[
 RowBox[{"xlist", " ", "=", " ", 
  RowBox[{"dumbCNRsolver", "[", 
   RowBox[{"f", ",", "0.0"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5247511201786427`*^9, 3.5247511333606663`*^9}, 
   3.5247511642731204`*^9, {3.524751232866441*^9, 3.524751233771242*^9}, {
   3.5247512640508957`*^9, 3.524751290149741*^9}, {3.524751330772213*^9, 
   3.524751332441416*^9}, {3.524751767214179*^9, 3.5247517935470257`*^9}, {
   3.524751826370683*^9, 3.524751856320335*^9}, 3.5247564251219587`*^9, 
   3.524756540889762*^9, {3.524756625691511*^9, 3.5247566287803164`*^9}, 
   3.524757487391225*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Select", " ", "which", " ", "of", " ", "the", " ", "two", " ", "exact", 
    " ", "roots", " ", "has", " ", "been", " ", "found"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"iexact", "=", "1"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Create", " ", "a", " ", "table", " ", "of", " ", "errors", " ", "in", 
     " ", "the", " ", "x", " ", "values", " ", "at", " ", "each", " ", 
     "iteration"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ex", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Abs", "[", 
        RowBox[{
         RowBox[{"xlist", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "-", 
         RowBox[{"xexact", "[", 
          RowBox[{"[", "iexact", "]"}], "]"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"Length", "[", "xlist", "]"}]}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"Create", " ", "a", " ", "list", " ", "of", " ", 
      SubscriptBox["u", "i"]}], "=", 
     RowBox[{"-", 
      RowBox[{"Log", "[", 
       SubscriptBox["e", "i"], "]"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"u", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"Log", "[", 
         RowBox[{"ex", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "ex", "]"}], "-", "1"}]}], "}"}]}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"Create", " ", "a", " ", "list", " ", "of", " ", 
      SubscriptBox["v", "i"]}], "=", 
     RowBox[{"-", 
      RowBox[{"Log", "[", 
       SubscriptBox["e", 
        RowBox[{"i", "+", "1"}]], "]"}]}]}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"v", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"Log", "[", 
         RowBox[{"ex", "[", 
          RowBox[{"[", 
           RowBox[{"i", "+", "1"}], "]"}], "]"}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "ex", "]"}], "-", "1"}]}], "}"}]}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"Combine", " ", "the", " ", "lists", " ", "into", " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["u", "i"], ",", 
       SubscriptBox["v", "i"]}], ")"}], " ", "data", " ", "pairs"}], " ", 
    "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"uvdat", "=", 
     RowBox[{"Transpose", "[", 
      RowBox[{"{", 
       RowBox[{"u", ",", "v"}], "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Perform", " ", "a", " ", "linear", " ", "fit", " ", "to", " ", "the", 
     " ", "data"}], "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"model", "=", 
     RowBox[{"LinearModelFit", "[", 
      RowBox[{"uvdat", ",", "ufit", ",", "ufit"}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Create", " ", "a", " ", "plot", " ", "showing", " ", "data", " ", "and", 
     " ", "the", " ", "fit"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Show", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Plot", "[", 
       RowBox[{
        RowBox[{"Normal", "[", "model", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"ufit", ",", 
          RowBox[{"Min", "[", "u", "]"}], ",", 
          RowBox[{"Max", "[", "u", "]"}]}], "}"}], ",", 
        RowBox[{"PlotStyle", "\[Rule]", "Thick"}]}], "]"}], ",", 
      RowBox[{"ListPlot", "[", 
       RowBox[{"uvdat", ",", 
        RowBox[{"PlotStyle", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"PointSize", "[", "0.02", "]"}], ",", "Red"}], "}"}]}]}], 
       "]"}], ",", 
      RowBox[{"AxesLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(u\), \
\(i\)]\)=-Log[\!\(\*SubscriptBox[\(e\), \(i\)]\)]\>\"", ",", 
         "\"\<\!\(\*SubscriptBox[\(v\), \
\(i\)]\)=-Log[\!\(\*SubscriptBox[\(e\), \(i + 1\)]\)]\>\""}], "}"}]}], ",", 
      RowBox[{"PlotLabel", "\[Rule]", 
       RowBox[{"Row", "[", 
        RowBox[{"{", 
         RowBox[{"\"\<\!\(\*SubscriptBox[\(v\), \(i\)]\)=\>\"", ",", 
          RowBox[{
           RowBox[{"Normal", "[", "model", "]"}], "/.", 
           RowBox[{
           "ufit", "\[Rule]", 
            "\"\<\!\(\*SubscriptBox[\(u\), \(i\)]\)\>\""}]}]}], "}"}], 
        "]"}]}]}], "]"}], "//", "Framed"}]}]}]], "Input",
 CellChangeTimes->{{3.524751421845173*^9, 3.5247515344305706`*^9}, {
   3.52475157977985*^9, 3.524751599045884*^9}, {3.524751629341137*^9, 
   3.524751650135974*^9}, {3.524752038279854*^9, 3.5247520562510853`*^9}, {
   3.524752111740383*^9, 3.5247521447968407`*^9}, {3.5247522010973396`*^9, 
   3.524752361262821*^9}, {3.524752395910482*^9, 3.524752447437372*^9}, {
   3.5247524824438343`*^9, 3.5247527195330505`*^9}, {3.524752820760428*^9, 
   3.5247528252376356`*^9}, {3.524752945342247*^9, 3.524752946496649*^9}, {
   3.524752976698302*^9, 3.5247530522180347`*^9}, {3.5247531034017243`*^9, 
   3.5247533886482253`*^9}, {3.524753839504617*^9, 3.524753950826413*^9}, 
   3.5247541166235037`*^9, {3.5247566689347873`*^9, 3.524756669387188*^9}, {
   3.524757488576827*^9, 3.524757493178835*^9}}],

Cell[BoxData[
 RowBox[{"Export", "[", 
  RowBox[{"\"\<ClassicalNewtonRaphsonConvergencePlot.png\>\"", ",", "%"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.524759157327157*^9, 3.5247591902276154`*^9}}],

Cell[TextData[{
 "Since the slope is approximatly 2, his Classical N-R solver has an \
approximately second-order rate of convergence.\n\nThe dumbCNRsolver has \
exactly the same results as the built-in solver, which is no surprise since \
the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " documentation indicates that the built-in solver is a classical NR solver. \
 By having written one ourselves, we have conducted a prudent verification \
test that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s algorithms are indeed working as advertised in their \
documentation!\n"
}], "Text",
 CellChangeTimes->{{3.5247541242207174`*^9, 3.524754193516039*^9}, {
  3.524756715469669*^9, 3.524756766778159*^9}, {3.52475681846105*^9, 
  3.5247568472743006`*^9}, {3.5247578812747164`*^9, 3.5247579053767586`*^9}, {
  3.524758105883911*^9, 3.5247582573289766`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Convergence analysis for the hand-coded modified NR solver\
\>", "Subsection",
 CellChangeTimes->{{3.524756389413496*^9, 3.5247564004115157`*^9}, {
  3.5247564948384814`*^9, 3.5247565109845095`*^9}, {3.524756789476199*^9, 
  3.524756790630601*^9}, {3.5247568583035197`*^9, 3.524756861829126*^9}}],

Cell["\<\
The modified NR solver is the same except that the slope is not re-evaluated \
at each iteration. Just the slope at the first guess is used.\
\>", "Text",
 CellChangeTimes->{{3.524755670642234*^9, 3.52475602722766*^9}, {
  3.5247560851817617`*^9, 3.524756126209834*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dumbMNRsolver", "[", 
   RowBox[{"f_", ",", "xguess_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"xlist", ",", 
      RowBox[{"tolerance", "=", 
       RowBox[{"1.0", " ", 
        SuperscriptBox["10", 
         RowBox[{"-", "8"}]]}]}], ",", "x", ",", "fprime"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"xlist", "=", 
      RowBox[{"{", "xguess", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"x", "=", "xguess"}], ";", "\[IndentingNewLine]", 
     RowBox[{"fprime", "=", 
      RowBox[{
       RowBox[{"f", "'"}], "[", "xguess", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"f", "[", "x", "]"}], "]"}], ">", "tolerance"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "=", 
         RowBox[{"x", "-", 
          FractionBox[
           RowBox[{"f", "[", "x", "]"}], "fprime"]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"xlist", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"xlist", ",", "x"}], "]"}]}], ";"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Return", "[", "xlist", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.524754978809018*^9, 3.524755000617857*^9}, {
  3.524755039571125*^9, 3.5247552465834885`*^9}, {3.5247552929623704`*^9, 
  3.524755426467405*^9}, {3.5247554761066923`*^9, 3.524755490131117*^9}, {
  3.524755551161624*^9, 3.5247555554516315`*^9}, {3.524755598492107*^9, 
  3.5247556423437843`*^9}, {3.5247556897210674`*^9, 3.5247556918582706`*^9}, {
  3.524756036587676*^9, 3.524756037476878*^9}, {3.5247561418566613`*^9, 
  3.524756167924307*^9}, {3.5247562015891666`*^9, 3.524756205270773*^9}, {
  3.5247563348446007`*^9, 3.5247563355622015`*^9}}],

Cell[BoxData[
 RowBox[{"xlist", " ", "=", " ", 
  RowBox[{"dumbMNRsolver", "[", 
   RowBox[{"f", ",", "0.0"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5247511201786427`*^9, 3.5247511333606663`*^9}, 
   3.5247511642731204`*^9, {3.524751232866441*^9, 3.524751233771242*^9}, {
   3.5247512640508957`*^9, 3.524751290149741*^9}, {3.524751330772213*^9, 
   3.524751332441416*^9}, {3.524751767214179*^9, 3.5247517935470257`*^9}, {
   3.524751826370683*^9, 3.524751856320335*^9}, 3.5247564251219587`*^9, 
   3.524756540889762*^9, {3.524756625691511*^9, 3.5247566287803164`*^9}, {
   3.524756793267006*^9, 3.524756794265407*^9}, 3.524757494816838*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Select", " ", "which", " ", "of", " ", "the", " ", "two", " ", "exact", 
    " ", "roots", " ", "has", " ", "been", " ", "found"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"iexact", "=", "1"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Create", " ", "a", " ", "table", " ", "of", " ", "errors", " ", "in", 
     " ", "the", " ", "x", " ", "values", " ", "at", " ", "each", " ", 
     "iteration"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ex", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Abs", "[", 
        RowBox[{
         RowBox[{"xlist", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "-", 
         RowBox[{"xexact", "[", 
          RowBox[{"[", "iexact", "]"}], "]"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"Length", "[", "xlist", "]"}]}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"Create", " ", "a", " ", "list", " ", "of", " ", 
      SubscriptBox["u", "i"]}], "=", 
     RowBox[{"-", 
      RowBox[{"Log", "[", 
       SubscriptBox["e", "i"], "]"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"u", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"Log", "[", 
         RowBox[{"ex", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "ex", "]"}], "-", "1"}]}], "}"}]}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"Create", " ", "a", " ", "list", " ", "of", " ", 
      SubscriptBox["v", "i"]}], "=", 
     RowBox[{"-", 
      RowBox[{"Log", "[", 
       SubscriptBox["e", 
        RowBox[{"i", "+", "1"}]], "]"}]}]}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"v", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"Log", "[", 
         RowBox[{"ex", "[", 
          RowBox[{"[", 
           RowBox[{"i", "+", "1"}], "]"}], "]"}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "ex", "]"}], "-", "1"}]}], "}"}]}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"Combine", " ", "the", " ", "lists", " ", "into", " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["u", "i"], ",", 
       SubscriptBox["v", "i"]}], ")"}], " ", "data", " ", "pairs"}], " ", 
    "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"uvdat", "=", 
     RowBox[{"Transpose", "[", 
      RowBox[{"{", 
       RowBox[{"u", ",", "v"}], "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Perform", " ", "a", " ", "linear", " ", "fit", " ", "to", " ", "the", 
     " ", "data"}], "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"model", "=", 
     RowBox[{"LinearModelFit", "[", 
      RowBox[{"uvdat", ",", "ufit", ",", "ufit"}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Create", " ", "a", " ", "plot", " ", "showing", " ", "data", " ", "and", 
     " ", "the", " ", "fit"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Show", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Plot", "[", 
       RowBox[{
        RowBox[{"Normal", "[", "model", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"ufit", ",", 
          RowBox[{"Min", "[", "u", "]"}], ",", 
          RowBox[{"Max", "[", "u", "]"}]}], "}"}], ",", 
        RowBox[{"PlotStyle", "\[Rule]", "Thick"}]}], "]"}], ",", 
      RowBox[{"ListPlot", "[", 
       RowBox[{"uvdat", ",", 
        RowBox[{"PlotStyle", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"PointSize", "[", "0.02", "]"}], ",", "Red"}], "}"}]}]}], 
       "]"}], ",", 
      RowBox[{"AxesLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(u\), \
\(i\)]\)=-Log[\!\(\*SubscriptBox[\(e\), \(i\)]\)]\>\"", ",", 
         "\"\<\!\(\*SubscriptBox[\(v\), \
\(i\)]\)=-Log[\!\(\*SubscriptBox[\(e\), \(i + 1\)]\)]\>\""}], "}"}]}], ",", 
      RowBox[{"PlotLabel", "\[Rule]", 
       RowBox[{"Row", "[", 
        RowBox[{"{", 
         RowBox[{"\"\<\!\(\*SubscriptBox[\(v\), \(i\)]\)=\>\"", ",", 
          RowBox[{
           RowBox[{"Normal", "[", "model", "]"}], "/.", 
           RowBox[{
           "ufit", "\[Rule]", 
            "\"\<\!\(\*SubscriptBox[\(u\), \(i\)]\)\>\""}]}]}], "}"}], 
        "]"}]}]}], "]"}], "//", "Framed"}]}]}]], "Input",
 CellChangeTimes->{{3.524751421845173*^9, 3.5247515344305706`*^9}, {
   3.52475157977985*^9, 3.524751599045884*^9}, {3.524751629341137*^9, 
   3.524751650135974*^9}, {3.524752038279854*^9, 3.5247520562510853`*^9}, {
   3.524752111740383*^9, 3.5247521447968407`*^9}, {3.5247522010973396`*^9, 
   3.524752361262821*^9}, {3.524752395910482*^9, 3.524752447437372*^9}, {
   3.5247524824438343`*^9, 3.5247527195330505`*^9}, {3.524752820760428*^9, 
   3.5247528252376356`*^9}, {3.524752945342247*^9, 3.524752946496649*^9}, {
   3.524752976698302*^9, 3.5247530522180347`*^9}, {3.5247531034017243`*^9, 
   3.5247533886482253`*^9}, {3.524753839504617*^9, 3.524753950826413*^9}, 
   3.5247541166235037`*^9, {3.5247566689347873`*^9, 3.524756669387188*^9}, {
   3.524757496486041*^9, 3.5247574971880417`*^9}}],

Cell[BoxData[
 RowBox[{"Export", "[", 
  RowBox[{"\"\<ModifiedNewtonRaphsonConvergencePlot.png\>\"", ",", "%"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.524759157327157*^9, 3.5247592124108543`*^9}}],

Cell["\<\
This modified NR has a slope of about 1.0 in its convergence plot, so it is \
approximately first-order.  

COMMENTARY: This solver required many more function evaluations and many more \
steps to converge, but that alone does not imply that it is necessarily more \
expensive computationally.  This solver could be the least expensive choice \
for functions that are so complicated that their derivative is highly \
expensive.  An example from constitutive modeling might be a function that \
outputs a Hencky strain tensor as a function of, say, a controlled component \
of stress.  Evaluating the derivative of Hencky strain is extremely \
expensive, so the Modified NR solver might be cheaper despite needing more \
evaluations.\
\>", "Text",
 CellChangeTimes->{{3.5247541242207174`*^9, 3.524754193516039*^9}, {
  3.524756715469669*^9, 3.524756766778159*^9}, {3.524756885884368*^9, 
  3.5247571854048944`*^9}, {3.524757920898786*^9, 3.524758040613396*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Convergence analysis for a hand-coded secant solver\
\>", "Subsection",
 CellChangeTimes->{{3.524756389413496*^9, 3.5247564004115157`*^9}, {
  3.5247564948384814`*^9, 3.5247565109845095`*^9}, {3.524756789476199*^9, 
  3.524756790630601*^9}, {3.5247568583035197`*^9, 3.524756861829126*^9}, {
  3.524758301289854*^9, 3.5247583071866646`*^9}}],

Cell["\<\
The following approximates the slope using secants, which is useful when it \
isn\[CloseCurlyQuote]t possible to evaluate derivatives\
\>", "Text",
 CellChangeTimes->{{3.524755670642234*^9, 3.52475602722766*^9}, {
  3.5247560851817617`*^9, 3.524756126209834*^9}, {3.524758312787074*^9, 
  3.5247583448295307`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dumbSECANTsolver", "[", 
   RowBox[{"f_", ",", "xguess1_", ",", "xguess2_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"xlist", ",", 
      RowBox[{"tolerance", "=", 
       RowBox[{"1.0", " ", 
        SuperscriptBox["10", 
         RowBox[{"-", "8"}]]}]}], ",", "x1", ",", "x2", ",", "f1", ",", "f2", 
      ",", "fprime"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"xlist", "=", 
      RowBox[{"{", "x1", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"x1", "=", "xguess1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"x2", "=", "xguess2"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"f", "[", "x1", "]"}], "]"}], ">", "tolerance"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"f1", "=", 
         RowBox[{"f", "[", "x1", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"f2", "=", 
         RowBox[{"f", "[", "x2", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"fprime", "=", 
         FractionBox[
          RowBox[{"f2", "-", "f1"}], 
          RowBox[{"x2", "-", "x1"}]]}], ";", "\[IndentingNewLine]", 
        RowBox[{"x2", "=", "x1"}], ";", "\[IndentingNewLine]", 
        RowBox[{"x1", "=", 
         RowBox[{"x1", "-", 
          FractionBox["f1", "fprime"]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"xlist", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"xlist", ",", "x1"}], "]"}]}], ";"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Return", "[", "xlist", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.524754978809018*^9, 3.524755000617857*^9}, {
  3.524755039571125*^9, 3.5247552465834885`*^9}, {3.5247552929623704`*^9, 
  3.524755426467405*^9}, {3.5247554761066923`*^9, 3.524755490131117*^9}, {
  3.524755551161624*^9, 3.5247555554516315`*^9}, {3.524755598492107*^9, 
  3.5247556423437843`*^9}, {3.5247556897210674`*^9, 3.5247556918582706`*^9}, {
  3.524756036587676*^9, 3.524756037476878*^9}, {3.5247561418566613`*^9, 
  3.524756167924307*^9}, {3.5247562015891666`*^9, 3.524756205270773*^9}, {
  3.5247563348446007`*^9, 3.5247563355622015`*^9}, {3.5247583533783455`*^9, 
  3.5247584563229265`*^9}, {3.524758508397818*^9, 3.5247586095483956`*^9}, {
  3.5247587088425703`*^9, 3.52475880554714*^9}, {3.524758853345624*^9, 
  3.5247589041393127`*^9}, {3.524759011327101*^9, 3.5247590210459185`*^9}}],

Cell[BoxData[
 RowBox[{"xlist", " ", "=", " ", 
  RowBox[{"dumbSECANTsolver", "[", 
   RowBox[{"f", ",", "0.0", ",", "0.5"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5247511201786427`*^9, 3.5247511333606663`*^9}, 
   3.5247511642731204`*^9, {3.524751232866441*^9, 3.524751233771242*^9}, {
   3.5247512640508957`*^9, 3.524751290149741*^9}, {3.524751330772213*^9, 
   3.524751332441416*^9}, {3.524751767214179*^9, 3.5247517935470257`*^9}, {
   3.524751826370683*^9, 3.524751856320335*^9}, 3.5247564251219587`*^9, 
   3.524756540889762*^9, {3.524756625691511*^9, 3.5247566287803164`*^9}, {
   3.524756793267006*^9, 3.524756794265407*^9}, 3.524757494816838*^9, {
   3.5247588213031673`*^9, 3.5247588381511974`*^9}, {3.524758988972262*^9, 
   3.5247589902202644`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Select", " ", "which", " ", "of", " ", "the", " ", "two", " ", "exact", 
    " ", "roots", " ", "has", " ", "been", " ", "found"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"iexact", "=", "1"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Create", " ", "a", " ", "table", " ", "of", " ", "errors", " ", "in", 
     " ", "the", " ", "x", " ", "values", " ", "at", " ", "each", " ", 
     "iteration"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ex", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Abs", "[", 
        RowBox[{
         RowBox[{"xlist", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "-", 
         RowBox[{"xexact", "[", 
          RowBox[{"[", "iexact", "]"}], "]"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"Length", "[", "xlist", "]"}]}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"Create", " ", "a", " ", "list", " ", "of", " ", 
      SubscriptBox["u", "i"]}], "=", 
     RowBox[{"-", 
      RowBox[{"Log", "[", 
       SubscriptBox["e", "i"], "]"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"u", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"Log", "[", 
         RowBox[{"ex", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "ex", "]"}], "-", "1"}]}], "}"}]}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"Create", " ", "a", " ", "list", " ", "of", " ", 
      SubscriptBox["v", "i"]}], "=", 
     RowBox[{"-", 
      RowBox[{"Log", "[", 
       SubscriptBox["e", 
        RowBox[{"i", "+", "1"}]], "]"}]}]}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"v", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"Log", "[", 
         RowBox[{"ex", "[", 
          RowBox[{"[", 
           RowBox[{"i", "+", "1"}], "]"}], "]"}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "ex", "]"}], "-", "1"}]}], "}"}]}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"Combine", " ", "the", " ", "lists", " ", "into", " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["u", "i"], ",", 
       SubscriptBox["v", "i"]}], ")"}], " ", "data", " ", "pairs"}], " ", 
    "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"uvdat", "=", 
     RowBox[{"Transpose", "[", 
      RowBox[{"{", 
       RowBox[{"u", ",", "v"}], "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Perform", " ", "a", " ", "linear", " ", "fit", " ", "to", " ", "the", 
     " ", "data"}], "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"model", "=", 
     RowBox[{"LinearModelFit", "[", 
      RowBox[{"uvdat", ",", "ufit", ",", "ufit"}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Create", " ", "a", " ", "plot", " ", "showing", " ", "data", " ", "and", 
     " ", "the", " ", "fit"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Show", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Plot", "[", 
       RowBox[{
        RowBox[{"Normal", "[", "model", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"ufit", ",", 
          RowBox[{"Min", "[", "u", "]"}], ",", 
          RowBox[{"Max", "[", "u", "]"}]}], "}"}], ",", 
        RowBox[{"PlotStyle", "\[Rule]", "Thick"}]}], "]"}], ",", 
      RowBox[{"ListPlot", "[", 
       RowBox[{"uvdat", ",", 
        RowBox[{"PlotStyle", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"PointSize", "[", "0.02", "]"}], ",", "Red"}], "}"}]}]}], 
       "]"}], ",", 
      RowBox[{"AxesLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(u\), \
\(i\)]\)=-Log[\!\(\*SubscriptBox[\(e\), \(i\)]\)]\>\"", ",", 
         "\"\<\!\(\*SubscriptBox[\(v\), \
\(i\)]\)=-Log[\!\(\*SubscriptBox[\(e\), \(i + 1\)]\)]\>\""}], "}"}]}], ",", 
      RowBox[{"PlotLabel", "\[Rule]", 
       RowBox[{"Row", "[", 
        RowBox[{"{", 
         RowBox[{"\"\<\!\(\*SubscriptBox[\(v\), \(i\)]\)=\>\"", ",", 
          RowBox[{
           RowBox[{"Normal", "[", "model", "]"}], "/.", 
           RowBox[{
           "ufit", "\[Rule]", 
            "\"\<\!\(\*SubscriptBox[\(u\), \(i\)]\)\>\""}]}]}], "}"}], 
        "]"}]}]}], "]"}], "//", "Framed"}]}]}]], "Input",
 CellChangeTimes->{{3.524751421845173*^9, 3.5247515344305706`*^9}, {
   3.52475157977985*^9, 3.524751599045884*^9}, {3.524751629341137*^9, 
   3.524751650135974*^9}, {3.524752038279854*^9, 3.5247520562510853`*^9}, {
   3.524752111740383*^9, 3.5247521447968407`*^9}, {3.5247522010973396`*^9, 
   3.524752361262821*^9}, {3.524752395910482*^9, 3.524752447437372*^9}, {
   3.5247524824438343`*^9, 3.5247527195330505`*^9}, {3.524752820760428*^9, 
   3.5247528252376356`*^9}, {3.524752945342247*^9, 3.524752946496649*^9}, {
   3.524752976698302*^9, 3.5247530522180347`*^9}, {3.5247531034017243`*^9, 
   3.5247533886482253`*^9}, {3.524753839504617*^9, 3.524753950826413*^9}, 
   3.5247541166235037`*^9, {3.5247566689347873`*^9, 3.524756669387188*^9}, {
   3.524757496486041*^9, 3.5247574971880417`*^9}}],

Cell[BoxData[
 RowBox[{"Export", "[", 
  RowBox[{"\"\<SecantSolverConvergencePlot.png\>\"", ",", "%"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.524759157327157*^9, 3.5247591902276154`*^9}, {
  3.5247592300856853`*^9, 3.5247592347812934`*^9}}],

Cell["\<\
With a convergence plot slope of 1.6, this secant solver is somewhere between \
1st and 2nd order accurate.\
\>", "Text",
 CellChangeTimes->{{3.5247541242207174`*^9, 3.524754193516039*^9}, {
  3.524756715469669*^9, 3.524756766778159*^9}, {3.524756885884368*^9, 
  3.5247571854048944`*^9}, {3.524757920898786*^9, 3.524758040613396*^9}, {
  3.5247590594213853`*^9, 3.5247591029454618`*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{707, 797},
WindowMargins->{{65, Automatic}, {6, Automatic}},
PrintingCopies->1,
PrintingPageRange->{32000, 32000},
PrintingOptions->{"Magnification"->1.,
"PaperOrientation"->"Portrait",
"PaperSize"->{612, 792}},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (February 23, 2011)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 199, 4, 137, "Title"],
Cell[781, 28, 374, 8, 83, "Text"],
Cell[1158, 38, 172, 3, 31, "Input"],
Cell[CellGroupData[{
Cell[1355, 45, 197, 3, 71, "Section"],
Cell[1555, 50, 141, 2, 29, "Text"],
Cell[1699, 54, 1016, 30, 72, "Input"],
Cell[2718, 86, 140, 3, 29, "Text"],
Cell[2861, 91, 273, 8, 31, "Input"],
Cell[CellGroupData[{
Cell[3159, 103, 152, 3, 36, "Subsection"],
Cell[3314, 108, 333, 7, 47, "Text"],
Cell[3650, 117, 981, 22, 72, "Input"],
Cell[4634, 141, 215, 5, 29, "Text"],
Cell[4852, 148, 99, 1, 31, "Input"],
Cell[4954, 151, 123, 2, 31, "Input"],
Cell[5080, 155, 350, 6, 47, "Text"],
Cell[5433, 163, 6020, 155, 412, "Input"],
Cell[11456, 320, 284, 5, 47, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11777, 330, 255, 5, 36, "Subsection"],
Cell[12035, 337, 1293, 25, 155, "Text"],
Cell[13331, 364, 1776, 43, 237, "Input"],
Cell[15110, 409, 158, 3, 29, "Text"],
Cell[15271, 414, 597, 10, 31, "Input"],
Cell[15871, 426, 5499, 142, 372, "Input"],
Cell[21373, 570, 201, 4, 31, "Input"],
Cell[21577, 576, 889, 18, 137, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[22503, 599, 306, 5, 36, "Subsection"],
Cell[22812, 606, 280, 5, 47, "Text"],
Cell[23095, 613, 1907, 45, 257, "Input"],
Cell[25005, 660, 643, 10, 31, "Input"],
Cell[25651, 672, 5501, 142, 372, "Input"],
Cell[31155, 816, 200, 4, 31, "Input"],
Cell[31358, 822, 970, 16, 155, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[32365, 843, 350, 6, 36, "Subsection"],
Cell[32718, 851, 324, 6, 29, "Text"],
Cell[33045, 859, 2536, 53, 373, "Input"],
Cell[35584, 914, 764, 12, 31, "Input"],
Cell[36351, 928, 5501, 142, 372, "Input"],
Cell[41855, 1072, 244, 5, 31, "Input"],
Cell[42102, 1079, 398, 7, 29, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
